package es.rickyepoderi.sample.test;

import es.rickyepoderi.sample.KeystorePasswordCallback;
import java.io.FileInputStream;
import java.net.URL;
import java.security.KeyStore;
import javax.net.ssl.KeyManagerFactory;
import javax.net.ssl.SSLContext;
import javax.net.ssl.TrustManagerFactory;
import javax.xml.ws.BindingProvider;
import org.apache.cxf.configuration.jsse.TLSClientParameters;
import org.apache.cxf.endpoint.Client;
import org.apache.cxf.frontend.ClientProxy;
import org.apache.cxf.transport.http.HTTPConduit;
import org.apache.cxf.ws.security.SecurityConstants;
import sample.rickyepoderi.es.ws.Echo;
import sample.rickyepoderi.es.ws.EchoService;

public class WSClient {

    /**
     * Initialize the SSLContext programmatically.
     * @param clientPKCS12 The pkcs12 keystore for the client
     * @return
     * @throws Exception 
     */
    public static SSLContext createSSLContext(String clientPKCS12) throws Exception {
        // load the cacerts file
        KeyStore cacerts = KeyStore.getInstance("JKS");
        FileInputStream cacertsFis = new FileInputStream("cacerts");
        cacerts.load(cacertsFis, "changeit".toCharArray());
        TrustManagerFactory tmf = TrustManagerFactory.getInstance(TrustManagerFactory.getDefaultAlgorithm());
        tmf.init(cacerts);
        // load the keystore file
        KeyStore keystore = KeyStore.getInstance("PKCS12");
        FileInputStream keystoreFis = new FileInputStream(clientPKCS12 + ".p12");
        keystore.load(keystoreFis, "Kiosko_00".toCharArray());
        KeyManagerFactory kmf = KeyManagerFactory.getInstance(KeyManagerFactory.getDefaultAlgorithm());
        kmf.init(keystore, "Kiosko_00".toCharArray());
        // create the ssl and initialize
        SSLContext ctx = SSLContext.getInstance("TLSv1.2");
        // do not configure to send the client certificate, not WSS is used
        //ctx.init(kmf.getKeyManagers(), tmf.getTrustManagers(), null);
        ctx.init(null, tmf.getTrustManagers(), null);
        return ctx;
    }
    
    public static void main(String... args) throws Exception {
        if (args.length != 2) {
            throw new IllegalAccessException("Usage: java WSClient client1 some_text_to_echo");
        }
        URL url = new URL("https://localhost:8443/wildfly-x509/echo-service/echo?wsdl");
        EchoService service = new EchoService(url);
        Echo echo = service.getEchoPort();
        // Properties for WS-Security configuration
        ((BindingProvider)echo).getRequestContext().put(SecurityConstants.CALLBACK_HANDLER, new KeystorePasswordCallback());
        ((BindingProvider)echo).getRequestContext().put(SecurityConstants.SIGNATURE_PROPERTIES, Thread.currentThread().getContextClassLoader().getResource(args[0] + ".properties"));
        ((BindingProvider)echo).getRequestContext().put(SecurityConstants.SIGNATURE_USERNAME, args[0]);
        // get the client and assign a specific SSLContext configuration
        Client client = ClientProxy.getClient(echo);
        HTTPConduit http = (HTTPConduit) client.getConduit();
        TLSClientParameters parameters = new TLSClientParameters();
        parameters.setSSLSocketFactory(createSSLContext(args[0]).getSocketFactory());
        http.setTlsClientParameters(parameters);
        System.err.println(echo.echo(args[1]));
    }
}
